/*
 *
 * Name:        Plugin_procs.c
 *
 * Description: These functions work hand-in-hand with those from
 *              OpenDVE_procs.c to provide OpenDVE messaging
 *              and clipboarding.  Tasks performed here are specific
 *              to OL, while OpenDVE_procs.c contains generic
 *              functions.
 *
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <Xm/Xm.h>

#include <stdio.h>
#include <sys/types.h>

#include <dve/dve.h>
#include <dve/clipboard.h>

#include "SamplePluginProtocol.h"
#include "mpApplicationShell.h"
#include "Plugin_procs.h"

void InsertNewTextMessage(DveUserHandle sender, char *text, int size);
void SendCurrentStatus(u_int32 messageType);
void QueryUserForCurrentStatus(DveUserHandle member);

extern Widget mpApplicationShell;
extern DveSession dveSession;
extern DveClipboard clipboard;

static const DvePort portNum=SAMPLE_PLUGIN_PORT;
static XmTextPosition wpr_position;

samplePluginMessageStructure curState;

/*
 * Insert the text string into our footer.
 */
void
mpSetFooter(const char *msg)
{
   XmString str=XmStringCreateSimple((char*)msg);
   XtVaSetValues(mpFooterLabel,
                 XmNlabelString,str,
                 NULL);
   XmStringFree(str);
}

/*
 * Display the contents of the DveClipboard in our scrolled
 * text window.  We have limited ourself text only in this 
 * application.
 */
void	
ProcessClipboardData(char *format, char *data, int length)
{
   char newMessage[1024];

   memset(newMessage, '\0', strlen(newMessage));
   sprintf(newMessage, "Clipboard Contents:\n");
   strncat(newMessage, data, length);
   strcat(newMessage, "\n");
   XmTextInsert(scrolledText1, wpr_position, newMessage);
   wpr_position = wpr_position + strlen(newMessage);
   XtVaSetValues(scrolledText1, XmNcursorPosition, wpr_position, NULL);
   XmTextShowPosition(scrolledText1, wpr_position);
}

/*
 * Call ShutdownOpenDVEClipboard to perform a dveClipClose().
 */
void
CleanupPlugin(void)
{
   ShutdownOpenDVEClipboard(clipboard);
}

/*
 * We've just initiated the conference and we need to determine
 * what our current setting should be.  Use
 * QueryUserForCurrentStatus() and our part->userHandle equals
 * the most senior conference member.
 */
void 
QueryPluginStatus(DveConfParticipant *part)
{
   fprintf(stderr, "We should ask Plugin status from %s@%s (%s)\n",
                    part->userName, part->hostName,
                    inet_ntoa(*(struct in_addr*)&((part->addr.v3))));
   QueryUserForCurrentStatus(part->userHandle);
}

/*
 * Received a dveEventTagMemberLeft.  The argument associated with
 * a dveEventTagMemberLeft is a DveConfParticipant.  Who left?
 */
void 
MemberLeft(DveConfParticipant member)
{
   char tmessage[128];
   sprintf(tmessage, "User Left the Conference - %s@%s (%s)",
                      member.userName, member.hostName,
                      inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
   mpSetFooter(tmessage);   
}

/*
 * Received a dveEventTagNewMember.  The argument associated with
 * a dveEventTagNewMember is a DveConfParticipant.  An invitation
 * has been sent to this user.  Please note, this member is
 * not active however until a MemberAccepted event is received.
 */
void 
MemberAdded(DveConfParticipant member)
{
   char tmessage[128];
   sprintf(tmessage, "User Added to Conference - %s@%s (%s)",
                      member.userName, member.hostName,
                      inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
   mpSetFooter(tmessage);   
}

/*
 * Invoked from PluginMessageCallback() found in OpenDVE_procs.c.
 * Determine the type of message received and then update our
 * Plugin settings to match those of other conferenced members.
 */
void 
ProcessPluginMessage(u_int32 messageType, void *data, DveUserHandle sender) 
{
   samplePluginMessageStructure message;
   char *tempPointer;

   memcpy(&message, data, sizeof(samplePluginMessageStructure));
 
   switch (messageType) {
      case TEMPERATURE_CHANGE:
         UpdateTemperature(ntohl(message.temperature));
         break;
      case MUSIC_CHANGE:
         UpdateMusic(ntohl(message.musicChoices));
         break;
      case DRINK_CHANGE:
         UpdateDrink(ntohl(message.drinkSelection));
         break;
      case TEXT_MESSAGE:
         tempPointer = ((samplePluginMessageStructure *)data)->message;
         InsertNewTextMessage(sender, tempPointer, ntohl(message.messageSize));
         break;
      case UPDATE_ALL_DATA:
         UpdateTemperature(ntohl(message.temperature));
         UpdateMusic(ntohl(message.musicChoices));
         UpdateDrink(ntohl(message.drinkSelection));
         break;
      case QUERY_PLUGIN_STATUS:
         SendCurrentStatus(UPDATE_ALL_DATA);
         break;
      default:
         fprintf(stderr, "Unknown Packet Type Received.\n");
         break;
   }
}

/*
 * Notify other conference members of our current settings.  We have
 * either made a change or have received a request to provide status.
 */
void
SendCurrentStatus(u_int32 messageType)
{
   samplePluginMessageStructure *msg;
 
   msg = (samplePluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(samplePluginMessageStructure));
   if (!msg)
      return;

   msg->protocolVersion = htonl(curState.protocolVersion);
   msg->temperature = htonl(curState.temperature);
   msg->musicChoices = htonl(curState.musicChoices);
   msg->drinkSelection = htonl(curState.drinkSelection);

   (void) SendPluginMessage(dveSession,
                            SAMPLE_PLUGIN_PORT, messageType, msg,
                            sizeof(samplePluginMessageStructure));
   return;
}

/*
 * Query an individual conference participant for their current settings--
 * most likely the senior conference member.
 */
void 	
QueryUserForCurrentStatus(DveUserHandle member)
{
   samplePluginMessageStructure *msg;
 
   msg = (samplePluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(samplePluginMessageStructure));
   if (!msg)
      return;

   msg->protocolVersion = VERSION;
 
   /*
    * Get the interface settings...
    */
   msg->temperature = 0;
   msg->musicChoices = 0;
   msg->drinkSelection = 0;
   msg->messageSize = 0;
 
   SendPluginMessageToMember(dveSession, SAMPLE_PLUGIN_PORT,
                             member, QUERY_PLUGIN_STATUS, msg,
                             sizeof(samplePluginMessageStructure));
   return;
}

/*
 * Send a text message to all conference participants.
 */
int
SendTextMessage(char *txtMessage, int length)
{
   samplePluginMessageStructure *msg;
   int rc;
 
   msg = (samplePluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(samplePluginMessageStructure) + length);
   if (!msg)
      return(-1);
 
   msg->protocolVersion = VERSION;
   msg->temperature = 0;
   msg->musicChoices = 0;
   msg->drinkSelection = 0;
   msg->messageSize = htonl(length);
   memcpy(msg->message, txtMessage, length);
 
   rc = SendPluginMessage(dveSession,
                          SAMPLE_PLUGIN_PORT, TEXT_MESSAGE, msg,
                          sizeof(samplePluginMessageStructure) + length);
 
   if (rc == 0)
      InsertNewTextMessage(0, txtMessage, length);
 
   return(rc);
}

/*
 * Callback associated with the temperature slider.  Call 
 * SendCurrentStatus() to notify other conference members.
 */
void
reportNewTemp(int newTemp)
{
   curState.temperature=newTemp;
   SendCurrentStatus(TEMPERATURE_CHANGE);
}

/*
 * Music callback.
 */
void
reportNewMusic(int newMusic)
{
   int bit=1<<newMusic;
   curState.musicChoices^=bit;
   SendCurrentStatus(MUSIC_CHANGE);
}

/*
 * Drink callback.
 */
void
reportNewDrink(int newDrink)
{
   curState.drinkSelection=newDrink;
   SendCurrentStatus(DRINK_CHANGE);
}

/*
 * Quit callback.  Shutdown the clipboard, close the OpenDVE
 * session and unregister with Xt.
 */
void
QuitCurrentSession(void)
{
   HidePlugin();
}

/*
 * Notified of a temperature change.  Adjust the temperature 
 * slider to reflect the change.
 */
void UpdateTemperature(int temp)
{
   XtVaSetValues(mpTempScale, 
                 XmNvalue, temp,
                 NULL);
   curState.temperature=temp;
}

/*
 * Used by UpdateMusic() to adjust the toggle settings.
 */
void updateToggle(int choice,
                  int whichbit,
                  Widget w)
{
   int flag=1<<whichbit;
   int flagbit=choice&flag;
   int res=(flagbit!=0);
 
   XtVaSetValues(w,
                 XmNset,res,
                 NULL);
}
 
/*
 * Notified of a music change.
 */
void UpdateMusic(int choice)
{
   updateToggle(choice,0,mpRockRadioButton);
   updateToggle(choice,1,mpBluesRadioButton);
   updateToggle(choice,2,mpCountryRadioButton);
 
   curState.musicChoices=choice;
}

/*
 * Notified of a drink change.
 */
void UpdateDrink(int drink)
{
   Widget w;
 
   switch (drink)
   {
      case 0:
         w=mpDrinkMenu_Coke;
         break;
 
      case 1:
         w=mpDrinkMenu_Pepsi;
         break;
 
      case 2:
         w=mpDrinkMenu_Beer;
         break;
 
      case 3:
         w=mpDrinkMenu_Tequila;
         break;
 
      case 4:
         w=mpDrinkMenu_Water;
         break;
 
      default:
         w=NULL;
         break;
   }

   if (w)
      XtVaSetValues(mpDrinkMenu,
                    XmNmenuHistory,w,
                    NULL);
   
   curState.drinkSelection=drink;
}

/*
 * We have received a text message from a conference member.
 * Display this message in our plugin.
 */
void
InsertNewTextMessage(DveUserHandle sender, char *text, int size)
{
   DveConfParticipant member;
   char newMessage[1024];
   char fullName[256];

   memset(newMessage, '\0', strlen(newMessage));
   if (sender == 0)
   {
      sprintf(newMessage, "Message From: ME!\n");
   }
   else if (dveGetOneMember(dveSession, sender, &member) < 0)
   {
      sprintf(newMessage, "Message From: Unknown User\n");
   }
   else
   {
      sprintf(newMessage, "Message From: %s@%s\n", member.userName, member.hostName);
   }
   strncat(newMessage, text, size);
   strcat(newMessage, "\n");
   XmTextInsert(scrolledText1, wpr_position, newMessage);
   wpr_position = wpr_position + strlen(newMessage);
   XtVaSetValues(scrolledText1, XmNcursorPosition, wpr_position, NULL);
   XmTextShowPosition(scrolledText1, wpr_position);
}

/*
 * SendpushButton callback.  Retrieve the text string from the
 * input text field and send it to our conference participants.
 * As seen above, SendTextMessage() calls SendPluginMessage() to 
 * make this happen.
 */
void
reportSendpushButton(void)
{
   char *text;

   if (text = (char *)XmTextGetString(mpMsgTextField))
   {
      if (SendTextMessage(text, strlen(text)) < 0)
      {
         mpSetFooter("Error Sending Message.");
      }
      else
      {
         mpSetFooter("Message Sent.");
         XmTextSetString(mpMsgTextField, "");
      }
      XtFree(text);
   }
}

/*
 * CopypushButton callback.
 */
void
reportCopypushButton(void)
{
   char *text;

   if (text = (char *)XmTextGetString(mpMsgTextField))
   {
      if (CopyDataToClipboard(clipboard, text, strlen(text), IS_TEXT_CLIPBOARD_ITEM) < 0)
      {
         mpSetFooter("Error Copying Clipboard.");
      }
      else
      {
         mpSetFooter("Copy Complete.");
      }
      XtFree(text);
   }
}

/*
 * If the DveClipboard contains something display it, otherwise
 * notify that the local clipboard is empty.
 */
void
reportShowpushButton(void)
{
   if (GetOpenDVEClipboard(clipboard, IS_TEXT_CLIPBOARD_ITEM) < 0)
   {
      XmTextInsert(scrolledText1, wpr_position, "Clipboard Empty!\n");
      wpr_position = wpr_position + strlen("Clipboard Empty!\n");
      XtVaSetValues(scrolledText1, XmNcursorPosition, wpr_position, NULL);
      XmTextShowPosition(scrolledText1, wpr_position);
   }
}

void
HidePlugin()
{
   XtPopdown(mpApplicationShell);
}
 
void
ShowPlugin()
{
   XtPopup(mpApplicationShell, XtGrabNone);
}
 
void
introduceYourself()
{
   curState.protocolVersion = VERSION;
   curState.temperature = 50;
   curState.musicChoices = 0;
   curState.drinkSelection = 0;

   /*
    * Start up OpenDVE.
    */
   if (InitializeOpenDVE(UxAppContext, SAMPLE_PLUGIN_PORT)==NULL){
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      exit(0);
   }

   /* 
    * Establish OpenDVE Clipboard.
    */
   if (InitializePluginClipboard() < 0){
      fprintf(stderr, "Initialization of OpenDVE Clipboard Failed.\n\7");
   }
}
